@@ -1,4 +1,4 @@
-# $Id: IRC-State.pm,v 3.8 2005/03/04 17:37:21 chris Exp $
+# $Id: IRC-State.pm,v 3.10 2005/03/14 10:16:24 chris Exp $
 #
 # POE::Component::IRC, by Dennis Taylor <dennis@funkplanet.com>
 #
@@ -10,7 +10,7 @@
 package POE::Component::IRC::State;
 
 use strict;
-use POE;
+use POE qw(Component::IRC::Plugin::Whois);
 use base qw(POE::Component::IRC);
 use vars qw($VERSION);
 
@@ -57,6 +57,9 @@ sub _create {
     POE::Component::Client::DNS->spawn( Alias => "irc_resolver" );
   }
 
+  # Plugin 'irc_whois' and 'irc_whowas' support
+  $self->plugin_add ( 'Whois', POE::Component::IRC::Plugin::Whois->new() );
+
   $self->{IRC_CMDS} =
   { 'rehash'    => [ PRI_HIGH,   'noargs',        ],
     'restart'   => [ PRI_HIGH,   'noargs',        ],
@@ -101,7 +104,7 @@ sub _create {
 
   # We need to register additional events with ourselves.
 
-  $self->{IRC_EVTS} = [ qw(ping join part kick nick mode quit 352 324 315 311 312 313 317 319 318 314 369) ];
+  $self->{IRC_EVTS} = [ qw(001 ping join part kick nick mode quit 352 324 315) ];
 
   my (@event_map) = map {($_, $self->{IRC_CMDS}->{$_}->[CMD_SUB])} keys %{ $self->{IRC_CMDS} };
 
@@ -180,6 +183,11 @@ sub _parseline {
 # Event handlers for tracking the STATE. $self->{STATE} is used as our namespace.
 # u_irc() is used to create unique keys.
 
+# Make sure we have a clean STATE when we first join the network.
+sub irc_001 {
+  delete ( $_[OBJECT]->{STATE} );
+}
+
 # Channel JOIN messages
 sub irc_join {
   my ($kernel,$self,$who,$channel) = @_[KERNEL,OBJECT,ARG0,ARG1];
